﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Threading;
using Produmex.Foundation;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Utilities;

// PmxWorkflowExecutionTypes.MAIN_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    public class WorkflowScript_MainWarehouseMenuFlowScript_RCI : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Private fields
        private CultureInfo m_defaultCultureInfo;
        private PmxOseCompany m_pmxOseCompany;

        // Input parameters 
        // <none>

        // Output parameters
        // <none>

        // Sub-flows
        private WorkflowInstanceBase m_logOnFlow;
        private bool m_logOnFlow_LogOnCanceled;

        // Menu options
        private class MenuOption
        {
            public string ButtonKey;
            public WorkflowInstanceBase Flow;

            public MenuOption( string buttonKey, WorkflowInstanceBase flow )
            {
                ButtonKey = buttonKey;
                Flow = flow;
            }
        };
        private List<MenuOption> m_MenuOptions = new List<MenuOption>();

        public WorkflowScript_MainWarehouseMenuFlowScript_RCI(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
            // Initialize required sub-flows
            m_logOnFlow = Create("LogOnScript");
            m_logOnFlow.BindParameters(
                new ReadWriteBinder<CultureInfo>("DefaultCultureInfo", this, "m_defaultCultureInfo"),
                new ReadWriteBinder<PmxOseCompany>("PmxOseCompany", this, "m_pmxOseCompany"),
                new ReadWriteBinder<bool>("LogOnCanceled", this, "m_logOnFlow_LogOnCanceled")
            );

            //m_MenuOptions.Add(new MenuOption("Sales RCI", Create("SalesMainFlowScript_RCI")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_LOGISTICS", Create("LogisticsMainFlowScript")));
           // m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PRODUCTION", Create("ProductionMainFlowScript")));


            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PURCHASE", Create("PurchaseMainFlowScript_RCI")));
            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_SALES", Create("SalesMainFlowScript_RCI")));
            m_MenuOptions.Add(new MenuOption("MSG_BUTTON_LOGISTICS", Create("LogisticsMainFlowScript_RCI")));
          //  m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PRODUCTION", Create("ProductionMainFlowScript")));

            //m_MenuOptions.Add( new MenuOption( "MSG_BUTTON_PICKING", Create( "PickingScript" ) ) );
            //m_MenuOptions.Add( new MenuOption( "MSG_BUTTON_MULTI_PICKING", Create( "PickingScript" ).BindParameters( new ReadOnlyBinder<bool>( "MultiPicking", true ) ) ) );

            //    m_MenuOptions.Add( new MenuOption( "MSG_BUTTON_AD_HOC_PICKING", Create( "AdHocPickingScript" ) ) );
            ////m_MenuOptions.Add(new MenuOption("MSG_BUTTON_SHIPPING", Create("ShippingFlow")));
            //    m_MenuOptions.Add(new MenuOption("MSG_BUTTON_SHIPPING", Create("ShippingScript")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PUT_AWAY", Create("PutAwayFlow")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_RETOUR_SALES", Create("SalesReturnScript")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_RETURN_PURCHASE", Create("PurchaseReturnFlow")));
            ////m_MenuOptions.Add(new MenuOption("OUDE AD HOC MOVE", Create("AdHocStockMovementFlow")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_AD_HOC_MOVE", Create("AdHocMoveScript")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_MOVE_ORDER", Create("MoveOrderScript")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_REPLENISH", Create("ReplenishFlow")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PICKING_FOR_PRODUCTION", Create("PickingForProductionScript")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_MOVE_TO_PRODUCTION_LINE_MENU", Create("MoveToProductionLineStandAloneScript")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PRODUCTION_RECEIPT", Create("ProductionReceiptScript")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_PRINT_ITEM_LABEL", Create("PrintItemLabelFlow")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_DIRECT_CYCLE_COUNTING", Create("BasicCycleCountingScript")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_CYCLE_COUNTING", Create("CycleCountingScript")));
            //m_MenuOptions.Add(new MenuOption("MSG_BUTTON_CHECKS", Create("ChecksScript")));
            //m_MenuOptions.Add(new MenuOption("Purchase return script", Create("PurchaseReturnScript")));

            foreach (MenuOption menuOption in m_MenuOptions)
            {
                List<string> list = new List<string>();
                list.Add(menuOption.ButtonKey);
                menuOption.Flow.BindParameters(
                    new ReadOnlyBinder<CultureInfo>("DefaultCultureInfo", this, "m_defaultCultureInfo"),
                    new ReadOnlyBinder<PmxOseCompany>("PmxOseCompany", this, "m_pmxOseCompany"),
                    new ReadOnlyBinder<List<string>>("MenuPath", list  )
                );
            }

        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Store menu options indexed by button key
            Dictionary<string, MenuOption> menuOptionsByButtonKey = new Dictionary<string, MenuOption>();
            foreach ( MenuOption menuOption in m_MenuOptions)
            {
                menuOptionsByButtonKey[menuOption.ButtonKey] = menuOption;
            }

            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;

            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            while ( true )
            {
            Step_Logon:

                // Run sub-flow LogOn
                base.StartSubFlowAndWait(m_logOnFlow, true);


                if ( m_logOnFlow_LogOnCanceled ) break;

                string initialErrorKey = null;

                int topMenuOptionIndex = 0;

            Step_SelectFunction:

                string[] buttonKeys = new string[ 6 ];
                int btnIdx = 0;
                for ( int i = topMenuOptionIndex; i < m_MenuOptions.Count; i++ )
                {
                    buttonKeys[ btnIdx ] = m_MenuOptions[ i ].ButtonKey;

                    if ( ++btnIdx >= 4 ) break;
                }

                ToolBarButtons toolBarButtons = ToolBarButtons.Stop;
                if ( topMenuOptionIndex > 0 ) toolBarButtons |= ToolBarButtons.Back;
                if ( topMenuOptionIndex + 4 < m_MenuOptions.Count ) toolBarButtons |= ToolBarButtons.Forward;

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                    m_defaultCultureInfo, BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_FUNCTION" + ( topMenuOptionIndex / 4 + 1 ).ToString(),
                    "AllowedButtons", buttonKeys,
                    "EnabledToolBarButtons", toolBarButtons,
                    "ScreenType", ScreenType.Menu
                    ));
                Message msg = WaitForMessage();
                initialErrorKey = null;
                if ( msg.Name.EndsWith(".ButtonClicked") )
                {
                    string buttonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");
                    MenuOption menuOption;
                    if ( menuOptionsByButtonKey.TryGetValue(buttonKey, out menuOption) )
                    {
                        try
                        {
                            base.StartSubFlowAndWait(menuOption.Flow, true);

                        }
                        catch ( Exception ex )
                        {
                            s_log.Error(String.Format("Exception in workflow behind button {0}", menuOption.ButtonKey), ex);

                            ProdumexError pmxError;
                            ex = ExtractExceptionAndPmxErrorForDisplay(ex, out pmxError);

                            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                m_defaultCultureInfo, BuildParamCollection(
                                "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, m_defaultCultureInfo),
                                "NoTranslationOfMessageKey", true,
                                "PmxError", pmxError,
                                "MessageAddendum", ReflectionHelper.CompactStackTrace( ex.StackTrace ),
                                "ShowButton", true,
                                "IsError", true
                                ));
                            msg = WaitForMessage();
                        }

                        goto Step_SelectFunction;
                    }
                }
                if ( msg.Name.EndsWith(".BackRequested") )
                {
                    topMenuOptionIndex = Math.Max(0, topMenuOptionIndex - 4);
                    goto Step_SelectFunction;
                }
                if ( msg.Name.EndsWith(".ForwardRequested") )
                {
                    topMenuOptionIndex += 4;
                    goto Step_SelectFunction;
                }
                if ( msg.Name.EndsWith(".StopRequested") )
                {
                    sboProviderService.LogOffWithSessionId(session.Id);
                    goto Step_Logon;
                }

                initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
                goto Step_SelectFunction;
            }
        }

        #endregion
    }
}
